#!/usr/bin/env python3
# This script is part of Acronis Backup Advanced for SAP HANA solution
# It is supported by Acronis only as part of the entire solution in accordance with http://www.acronis.com/support/eula-corp.html
# By using this script you accept the terms of the EULA http://www.acronis.com/support/eula-corp.html
# Modification or use outside of the solution is done at your own risk and will not be supported by Acronis
# Contact http://www.acronis.com/en-us/support/ with any questions or concerns
# Copyright (C) 2019 Acronis

import sys
import logging
import argparse

from util import Credentials, get_credentials, save_credentials, logger, encrypt, PathConfig, save_path_config

def print_credential(credential):
    print("Instance number: " + credential._instance_number +
        "\nDatabase: " + credential._database)
    if credential._key:
        print("Key:" + credential._key)
    else:
        print("Username: " + credential._username)
    print ("\n")

def check_null(arg, output):
    if not arg:
        logger.info(output)
        sys.exit(1)

def check_common_arugment(args):
    check_null(args.instance_number, 'Invalid instance number')
    check_null(args.database_name, 'Invalid database')

#------------------------------------------------------
# Operations
#------------------------------------------------------
def add_instance(instance_number, database, user = "", password = "", key = ""):
    new_credential = Credentials(instance_number, database, user, encrypt(password), key)

    cred_array = []
    cred_check = get_credentials()
    if cred_check is not None:
        cred_array = cred_check
    else:
        logger.info("Generating new config file")

    #Check for duplicated number and name
    for cred in cred_array:
        if (cred._instance_number == new_credential._instance_number and
                cred._database == new_credential._database):
            logger.info("Duplicate instance detected, replacing instance")
            cred_array.remove(cred)
            break

    cred_array.append(new_credential)
    save_credentials(cred_array)
    print("Successfully added Instance")

def list_instance():
    cred_array = get_credentials()
    if cred_array is None:
        logger.error("Config file not found")
        sys.exit(1)

    for cred in cred_array:
        print_credential(cred)

def get_instance(instance_number, database):
    cred_array = get_credentials()
    if cred_array is None:
        logger.error("Config file not found")
        sys.exit(1)

    for cred in cred_array:
        if(cred._instance_number == instance_number and
                cred._database == database):
            print_credential(cred)
            return

    logger.error("Instance number not found - " + instance_number)

def remove_instance(instance_number, database):
    cred_array = get_credentials()
    if cred_array is None:
        logger.error("Config file not found")
        sys.exit(1)

    for cred in cred_array:
        if(cred._instance_number == instance_number and
                cred._database == database):
            cred_array.remove(cred)
            save_credentials(cred_array)
            logger.info("Successfully removed instance")
            return

    logger.error("Instance not found - Instance Number: " + instance_number + " Database: " + database)

#------------------------------------------------------
# Parsing
#------------------------------------------------------
def list_and_return_it(element, theList):
    theList.append(element)
    return element

def add_parser(parsers, name, theHelp, parsersList):
    return list_and_return_it(parsers.add_parser(name, help=theHelp), parsersList)

ARG_ADD = "ADD"
ARG_GET = "GET"
ARG_REMOVE = "REMOVE"
ARG_LIST = "LIST"

SET_INSTALL_PATH = "SET_INSTALL_PATH"
#------------------------------------------------------
# Main
#------------------------------------------------------
def main():
     #Parse agruments
    parser = argparse.ArgumentParser(description='Acronis SAP HANA config')
    subparsers = parser.add_subparsers(dest='which')
    allParsers = []

    parser_add = add_parser(subparsers, ARG_ADD, 'Add instance credential', allParsers)
    add_parser(subparsers, ARG_GET, 'Get instance credential', allParsers)
    add_parser(subparsers, ARG_REMOVE, 'Remove instance credential', allParsers)

    for theParser in allParsers:
        theParser.add_argument('-i', '--instance_number', help='The instance number', required=True)
        theParser.add_argument('-d', '--database_name', help='The name of the database', required=True)

    user_key_mutex_group = parser_add.add_mutually_exclusive_group(required=True)
    user_key_mutex_group.add_argument('-U', '--user_store_key', help = 'The keyprefix of the secure user store')
    user_key_mutex_group.add_argument('-u', '--user', help='The user to the database')

    parser_add.add_argument('-p', '--password', help='The password to the database')

    add_parser(subparsers, ARG_LIST, 'List instance credential', allParsers)

    #parser for getting sap service file
    set_sap_parser = add_parser(subparsers, SET_INSTALL_PATH, 'Set location where SAP HANA is installed', allParsers)
    set_sap_parser.add_argument('--install_path', help='The install path for SAP HANA', required=True)

    args = parser.parse_args()

    if args.which == ARG_ADD:
        check_common_arugment(args)
        if args.user:
            logger.info('Adding instance')
            check_null(args.user, 'Invalid user')
            check_null(args.password, 'Invalid password')
            add_instance(args.instance_number, args.database_name, user = args.user, password = args.password)
        else:
            #no user detect, using secure key method
            logger.info('Adding secure user key: ' + args.user_store_key)
            add_instance(args.instance_number, args.database_name, key = args.user_store_key)
    elif args.which == ARG_LIST:
        logger.info('Listing instances:\n')
        list_instance()
    elif args.which == ARG_GET:
        check_common_arugment(args)
        logger.info('Getting instance:\n')
        get_instance(args.instance_number, args.database_name)
    elif args.which == ARG_REMOVE:
        check_common_arugment(args)
        logger.info ('Removing instance: ' + args.instance_number)
        remove_instance(args.instance_number, args.database_name)
    elif args.which == SET_INSTALL_PATH:
        save_path_config(PathConfig(args.install_path))
    else:
        parser.parse_args(['-h'])

if __name__ == '__main__':
    main()
