#!/usr/bin/env python3
# This script is part of Acronis Backup Advanced for SAP HANA solution
# It is supported by Acronis only as part of the entire solution in accordance with http://www.acronis.com/support/eula-corp.html
# By using this script you accept the terms of the EULA http://www.acronis.com/support/eula-corp.html
# Modification or use outside of the solution is done at your own risk and will not be supported by Acronis
# Contact http://www.acronis.com/en-us/support/ with any questions or concerns
# Copyright (C) 2019 Acronis

import subprocess
import os
import errno
import re
import yaml
import logging
import base64
import sys
import datetime

usrsap = '/usr/sap'

VERBOSE = True
#------------------------------------------------------
# Debugging section
#------------------------------------------------------

def check_and_create_directory(path):
    #create directory if it doesn't exist code
    if not os.path.exists(os.path.dirname(path)):
        try:
            os.makedirs(os.path.dirname(path))
        except OSError as exc: # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise

log_dir = '/var/lib/Acronis/Logs/SAPHANA/'
check_and_create_directory(log_dir)
time = datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
log_name = time + '_hana.log'
log_path = log_dir + log_name

i = 0
while os.path.exists(log_path):
    i+=1
    log_path = log_dir + time + '_hana(' + str(i) + ').log'

logger = logging.getLogger('logger')
logger.setLevel(logging.INFO)

# create a file handler
handler = logging.FileHandler(log_path)
handler.setLevel(logging.INFO)

# add the handlers to the logger
logger.addHandler(handler)
if VERBOSE:
    logger.addHandler(logging.StreamHandler())

formatter = logging.Formatter('%(asctime)s - %(filename)s - %(levelname)s - %(funcName)s - %(message)s')
handler.setFormatter(formatter)

def debug_print(string):
    if VERBOSE:
        print(string)

def encrypt(string):
    return base64.b64encode(string.encode('utf-8')).decode('ascii')

def decrypt(string):
    return base64.b64decode(string.encode('utf-8')).decode('ascii')

#------------------------------------------------------
# Credentials section
#------------------------------------------------------
config_directory = "/var/lib/Acronis/SAPHANA/Config/"

config_name = 'config.yaml'
config_path = config_directory + config_name

pathconfig_name = 'path.yaml'
pathconfig_path = config_directory + pathconfig_name

class Credentials():
    def __init__(self, instance_number, database, username, password, key = ""):
        self._instance_number = instance_number
        self._database = database
        self._username = username
        self._password = password
        self._key = key

def get_credentials():
    return get_yaml_file(config_path)

def save_credentials(output):
    save_yaml_file(output, config_path, config_directory)

class PathConfig():
    def __init__(self, install_path):
        self._install_path = install_path

def get_path_config():
    return get_yaml_file(pathconfig_path)

def save_path_config(output):
    save_yaml_file(output, pathconfig_path, config_directory)

def get_yaml_file(path):
    if not os.path.isfile(path):
        return None

    stream = open(path, 'r')
    return yaml.load(stream)

def save_yaml_file(output, path, directory):
    check_and_create_directory(directory)
    stream = open(path, 'w')
    yaml.dump(output, stream)
    os.chmod(path, 0o600)
#------------------------------------------------------
# SAP section
#------------------------------------------------------

#Reading of sapservices file
#Code mostly taken and ported to python from "sapinit" script
def read_sapservices(sapservices):

    pathconfig = get_path_config()
    if pathconfig is None:
        #Set default sap service file path if not found
        save_path_config(PathConfig(usrsap))
        pathconfig = get_path_config()

    sapservices_file = pathconfig._install_path + "/sapservices"

    if not os.path.isfile(sapservices_file):
        logger.error("SAP HANA service file not found, please run config and set the installation path for SAP HANA")
        sys.exit(1)

    subexp="pf\=.*|-u *[[:alnum:]]{6}|-D"
    expression= pathconfig._install_path + "/[[:alnum:]]{3}/.*sapstartsrv *%s *%s *%s" % (subexp, subexp, subexp)

    #Count amount of line in 'sapservices' file
    out = subprocess.Popen(["cat", sapservices_file], stdout=subprocess.PIPE)
    out = subprocess.Popen(["wc", "-l"], stdin=out.stdout, stdout=subprocess.PIPE, universal_newlines=True)
    lines =  out.communicate()[0]

    #Check each line against regex expression
    l, i = 0, 1
    while i <= int(lines) :
        out = subprocess.Popen(["head", "-" + str(i), sapservices_file], stdout=subprocess.PIPE)
        out = subprocess.Popen(["tail", "-1"], stdin=out.stdout, stdout=subprocess.PIPE)
        out = subprocess.Popen(["grep", "-E", expression], stdin=out.stdout, stdout=subprocess.PIPE, universal_newlines=True)
        output = out.communicate()[0]

        if output:
            sapservices.insert(l,output)
            l+=1
        i+=1

class CommandHandler():
    def __init__(self, service, db_command, credentials):
        path_and_instance = self.get_path_and_instance(service)
        self._hdbpath = path_and_instance[0]
        self._instance_num = path_and_instance[1]

        #matching instance number from config file to the list from sap
        self._commands = []
        for cred in credentials:
            if self._instance_num == cred._instance_number:
                if cred._key:
                    self._commands.append([self._hdbpath + '/hdbsql', '-a', '-x', '-j', '-d', cred._database, '-U', cred._key, '-i', cred._instance_number, db_command])
                else:
                    self._commands.append([self._hdbpath + '/hdbsql', '-a', '-x', '-j', '-d', cred._database, '-u', cred._username, '-p', decrypt(cred._password), '-i', cred._instance_number, db_command])

        if not self._commands:
            logger.error('No valid credential found')
            sys.exit(1)

    def get_path_and_instance(self, service):
        result = re.search("LD_LIBRARY_PATH=(.+):.*pf=(\S+)", service)

        if not result:
            logger.error("Path not found!")
            sys.exit(1)

        #Getting the path
        hdbpath = result.groups(0)[0]

        #Getting the instance information
        textfile = open(result.groups(0)[1], 'r')
        filetext = textfile.read()
        textfile.close()
        reg = re.search("SAPSYSTEM = (.+)", filetext)
        if not reg:
            logger.error("Path not found!")
            sys.exit(1)
        instanceNum = reg.groups(0)[0]

        logger.info("Instance Number: " + instanceNum)
        logger.info("HDBpath : " + hdbpath + "\n")

        return [hdbpath, instanceNum]

    def set_db_command(self, db_command):
        for comm in self._commands:
            #Last element is the database command
            db_command_index = len(comm) - 1
            comm[db_command_index] = db_command

    def get_database_position(self):
        DATABASE_POSITION = 5
        return DATABASE_POSITION